from enum import Enum


class MT5TimeFrame(Enum):
    """ 
    Metatrader 5 dataframes
    """
    M1 = 1  # 1 minute
    M5 = 5  # 5 minutes
    M15 = 15  # 15 minutes
    M30 = 30  # 30 minutes
    Hourly = 60  # 1 hourly
    Daily = 1440  # Daily
    Weekly = 10080  # Weekly
    Montly = 43200  # Monthly



import time
import logging
import pandas as pd
from datetime import datetime, timedelta
from pyomt5.api import MT5TimeFrame
from pyomt5.api import MetatraderCom, ConnectionTimeoutError

log = logging.getLogger(__name__)


class StockPriceHistory():
    def __init__(self, use_cache=False, **kw):
        self.use_cache = use_cache
        self.price_cache = dict()

        self.number_of_retry = 10
        if kw.get('retry'):
            self.number_of_retry = kw.get('retry')

        self.request_timeout = 1.5
        if kw.get('timeout'):
            self.request_timeout = kw.get('timeout')

    def _convert_to_dataframe(self, data):
        df = pd.DataFrame.from_dict(data)

        df.close = pd.to_numeric(df.close)
        df.open = pd.to_numeric(df.open)
        df.low = pd.to_numeric(df.low)
        df.high = pd.to_numeric(df.high)
        df.volume = pd.to_numeric(df.volume)
        df.date = pd.to_datetime(df.date,
                                 format='%Y.%m.%d %H:%M',
                                 errors='ignore')

        return df

    def _prepare_data(self, symbol, data):
        df = self._convert_to_dataframe(data)

        df['symbol'] = symbol

        return df.reset_index().\
            sort_values(by=["symbol", "date"],
                        ascending=True).set_index(["symbol", "date"]).\
            drop(columns=["index"])

    def get_price_from(self, symbol: str, from_date: datetime,
                       to_date: datetime, timeframe: MT5TimeFrame) -> list:
        """ Get prices from history data and return it 
            in a pandas DataFrame format
            Args:
                symbol (str): symbol of stock
                timeframe (MT5TimeFrame): interval between each price-time
                periods: amount of data that'll be returned
            Returns:
                list: a list with all data in the OHLC format 
                Ex:
                                    close   high    low   open     volume
                symbol date
                PETR4  2018-09-20    19.18  19.64  19.07  19.56   56002300
                       2018-09-21    19.44  19.60  19.26  19.45   58278000
        """
        end_date = to_date
        start_date = from_date

        c = MetatraderCom()
        nRetry = 0
        price_history = None
        while nRetry < self.number_of_retry:
            try:

                if self.use_cache and (symbol in self.price_cache):
                    log.debug(f"Getting data for {symbol} [using cache]")
                    price_history = self.price_cache[symbol]
                    break

                log.debug(f"Getting data for {symbol}")
                price_history = c.get_historical(symbol, timeframe.value,
                                                 start_date, end_date)

                self.price_cache[symbol] = price_history
                break

            except ConnectionTimeoutError:
                nRetry += 1
                log.debug(
                    f"Connection Failed. Waiting for reconnection.. Retry #{nRetry}"
                )
                time.sleep(self.request_timeout)

        return self._prepare_data(symbol, price_history)



import logging
import zmq
from .timeframe import MT5TimeFrame

log = logging.getLogger(__name__)

DATA_NOT_FOUND = 'NO DATA'


class DataNotFoundError(Exception):
    pass


class ConnectionTimeoutError(Exception):
    pass


class MetatraderCom():
    def __remote_send(self, socket, data):
        try:
            socket.send_string(data)
            msg = socket.recv_string()
            return (msg)
        except zmq.Again:
            log.debug("Waiting for PUSH from MetaTrader 5..")

    def __get_socket(self):
        context = zmq.Context()

        # Create REQ Socket
        reqSocket = context.socket(zmq.REQ)
        reqSocket.setsockopt(zmq.RCVTIMEO, 500)
        reqSocket.connect("tcp://localhost:5555")

        return reqSocket

    def get_historical(self, symbol: str, timeframe: MT5TimeFrame, start,
                       end) -> list:
        """ Get prices from Metatrader 5
            Args:
                symbol (str): symbol of stock
                timeframe (MT5TimeFrame): interval between each price-time
                start: start datetime of period
                end:   end datetime of period
            Returns:
                list: a list with all data in the OHLC format
                Ex:
                    [{'date': '2018.09.20 00:00', 'open': ' 19.5600',
                     'low': '19.0700', 'high': '19.6400', 'close':
                     '19.1800', 'volume': '56002300'}]

            Usage:
                start_date = datetime(year=2019, month=4, day=1,
                                      hour=10, minute=0, second=0)
                end_date = datetime(year=2019, month=4, day=3,
                                       hour=18, minute=0, second=0)

                c = MetatraderCom()
                c.get_historical('PETR4', M15, start_date, end_date)
        """
        reqSocket = self.__get_socket()

        start_date = start.strftime("%Y.%m.%d %H:%M:%S")
        end_date = end.strftime("%Y.%m.%d %H:%M:%S")
        data = self.__remote_send(
            reqSocket, f"DATA|{symbol}|{timeframe}|{start_date}|{end_date}")

        if data is None:
            reqSocket.close(100)
            raise ConnectionTimeoutError()

        if data == DATA_NOT_FOUND:
            raise DataNotFoundError()

        results = []
        for item in data.split('|'):
            splited_item = item.split(",")
            if len(splited_item) > 1:
                vdate, vopen, vlow, vhigh, \
                    vclose, vtick, vvolume = splited_item

            results.append(
                dict(date=vdate,
                     open=vopen,
                     low=vlow,
                     high=vhigh,
                     close=vclose,
                     volume=vvolume))
        return results[:-1]



import sys
import os
from setuptools import setup, find_packages
from codecs import open

here = os.path.abspath(os.path.dirname(__file__))
try:
    with open(
            'README.md',
            'r',
    ) as f:
        readme = f.read()
except IOError:
    readme = 'pyOMT5 - Python Open Metatrader 5 module'

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()

setup(
    name='pyOMT5',
    version='0.0.27',
    author='Paulo Rodrigues',
    author_email='paulorodriguesxv@gmail.com',
    license='MIT',
    description='pyOMT5 - Python Open Metatrader 5 module',
    long_description=readme,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Office/Business :: Financial :: Investment',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
    ],
    url='https://github.com/paulorodriguesxv/pyomt5',
    install_requires=['pyzmq', 'pandas'],
    tests_require=[],
    extras_require={
        'pandas': ['pandas'],
    },
    keywords=['stocks', 'market', 'finance', 'metatrader', 'quotes', 'shares'],
    packages=find_packages(exclude=['tutorial']),
    package_data={
        'pyomt5': [],
    })
